import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import java.text.DecimalFormat;
import java.util.*;

public class AttributeCell extends JComponent {
	private static final long serialVersionUID = 1L;

	static public final int DEFAULT_HEIGHT = 64;
    
    private Vector entryList;
    private String attributeName;
    private int colWidth = ValueChart.DEFAULT_COL_WIDTH;
    private Color color = Color.red;
    private ValueChart chart;
    private String units;
    private double threshold;
    AttributeDomain domain;
    JPopupMenu domainPopup;
    JPopupMenu entryPopup;
    JMenuItem entryPopupMenuItem;
    ContGraph cg;
    DiscGraph dg;
    
    JObjective obj;    
        
    public AttributeCell(ValueChart chart, AttributeDomain domain) {
        MouseHandler mouseHandler = new MouseHandler();
        addMouseListener(mouseHandler);
        addMouseMotionListener(mouseHandler);
        addComponentListener(new ResizeHandler());
        this.chart = chart;
        this.domain = domain;
    }    
   
    //temp
    public String getName(){
    	return attributeName;
    }
    
    public void setDomain(AttributeDomain ad){
    	domain = ad;
    }
 
    public double getOverallRatio() { // return overallRatio;
        Container container = getParent();
        double ratio = 1.0;
        while (container instanceof BaseTableContainer)
        { ratio *= ((BaseTableContainer)container).getHeightRatio();//-
          container = container.getParent().getParent();
        }
        return ratio;
    }
    
    public void setOverallRatio(double r){
    }
    
    public Color getColor(){ 
    	return color;
    }
    
	public void setColor(Color c){ 
    	color = c;
    	float[] rgb = c.getRGBColorComponents(null);
    	new Color(rgb[0]/3, rgb[1]/3, rgb[2]/3);
    }
    
    public String getUnits(){ 
    	return units;
    }
    
    public void setUnits(String name){ 
    	units = name;
    }
    
    //-sets the entry list for the cell
    public void setEntryList(String name, Vector list) {
        attributeName = name;
        entryList = list;
        Dimension dim =
        new Dimension(list.size()*colWidth, DEFAULT_HEIGHT);//- & rev xy
        setPreferredSize(dim);
        setSize(dim);
        dim.height = 1000000;//-w->h
        setMaximumSize(dim);
    }
        
    public void setSize(int w, int h) {
        super.setSize(w, h);        
    }
    
    public int numValues(){ 
    	return entryList.size();
    }
       
    //used by DisplayPanel to get a list of each entry's value
    public double[] getWeights() {
        double[] weights = new double[entryList.size()];
        int i = 0;
        for (Iterator it=entryList.iterator(); it.hasNext(); ){
        	AttributeValue value = ((ChartEntry)it.next()).attributeValue(attributeName);
        	if (value != null)
        		weights[i] = value.weight();
        	else
        		weights[i] = 0;
        	i++;
        }
        return weights;
    }
   
    public JPopupMenu getDomainPopup() {
        if (domainPopup == null)
        	makeHelpPopups(domain);        
        return domainPopup;
    }
    
    public JPopupMenu getEntryPopup() {
        if (entryPopup == null){ 
        	makeHelpPopups(domain);
        }
        return entryPopup;
    }
    
    private void makeHelpPopups(AttributeDomain domain) {
        domainPopup = new JPopupMenu();		
        String best = "";
        String worst = "";
        DecimalFormat df = obj.decimalFormat;
        if (domain.getType()==AttributeDomain.DISCRETE){
        	DiscreteAttributeDomain dd = (DiscreteAttributeDomain)domain;
		 	String elt[] = dd.getElements();
		 	double wt[] = dd.getWeights();
			 for (int j=0; j<wt.length; j++){
				if (wt[j]==0.0)
					worst = elt[j];
			 	if (wt[j]==1.0)
			 		best = elt[j];
			 }
		 }
		 else{
		 	ContinuousAttributeDomain cd = (ContinuousAttributeDomain)domain;
		 	double kt[] = cd.getKnots();
		 	double wt[] = cd.getWeights();
			 for (int j=0; j<wt.length; j++){	
			 	if (wt[j]==0.0)
			 		worst = df.format(kt[j]) + " " + units;
			 	if (wt[j]==1.0)
			 		best = df.format(kt[j]) + " " + units;
			 }
		 }		 
/*        if (domain.getType() == AttributeDomain.DISCRETE){ 
        	DiscreteAttributeDomain dd = (DiscreteAttributeDomain)domain;
        	String[] elements = dd.getElements();
        	String msg = null;
        	for (int i=0; i<elements.length; i++){ 
        		msg = elements[i] + " " + dd.weight(elements[i]);
            domainPopup.add(msg);
        	}
        }
        else{ 
        	ContinuousAttributeDomain cd = (ContinuousAttributeDomain)domain;
        	String rangeMsg = "[" + cd.getMin() + ", " + cd.getMax() + "]";
        	if (units != null)
        		rangeMsg += " " + units;
          
        	rangeMsg += " -> [" + cd.weight(cd.getMin()) + ", " +
			cd.weight(cd.getMax()) + "]";
        	domainPopup.add(rangeMsg);
        }*/
        String msg = "";
        msg = "BEST: " + best; 
        domainPopup.add(msg);
        msg = "WORST: " + worst; 
        domainPopup.add(msg);
        domainPopup.setSelectionModel(new NullSelectionModel());        
        entryPopup = new JPopupMenu();
        entryPopupMenuItem = new JMenuItem();
        entryPopup.add(entryPopupMenuItem);
        entryPopup.setSelectionModel(new NullSelectionModel());
    }
    //This is added to display utility graph
    public void getUtility(AttributeDomain domain){
        if (domain.getType() == AttributeDomain.DISCRETE) {
            DiscreteAttributeDomain dd =
            (DiscreteAttributeDomain)domain;
            new DiscreteUtilityGraph(chart, dd, dd.getElements(), dd.getWeights(), attributeName, null, this);
            
        }
        else
        { ContinuousAttributeDomain cd = (ContinuousAttributeDomain)domain;
          new ContinuousUtilityGraph(chart, cd, cd.getKnots(), cd.getWeights(), getUnits(), attributeName, null, this);
        }
    }    
    
    //This will make function call to get utility graph
    public void makeUtility(AttributeDomain domain){
    	if (domain.getType() == AttributeDomain.DISCRETE) {
            DiscreteAttributeDomain dd =
            (DiscreteAttributeDomain)domain;
            new DiscreteUtilityGraph(chart, dd, dd.getElements(), dd.getWeights(), attributeName, null, this);
            
        }
        else{ 
        	ContinuousAttributeDomain cd = (ContinuousAttributeDomain)domain;
        	new ContinuousUtilityGraph(chart, cd, cd.getKnots(), cd.getWeights(), getUnits(), attributeName, null, this);
        }
    }
    
    public ContGraph makeContGraph(AttributeDomain domain){    
    	ContinuousAttributeDomain cd = (ContinuousAttributeDomain)domain;
    	cg = new ContGraph(chart, cd, cd.getKnots(), cd.getWeights(), getUnits(), attributeName, color, this);
    	return cg;
    }
    
    public DiscGraph makeDiscGraph(AttributeDomain domain){
        DiscreteAttributeDomain dd = (DiscreteAttributeDomain)domain;
        dg = new DiscGraph(chart, dd, dd.getElements(), dd.getWeights(), attributeName, color);
        return dg;
    }
        
    AttributeDomain getDomain(){ 
    	return domain;
    }
    
    public int getColWidth(){ 
    	return colWidth;//-
    }
    
    public void setColWidth(int w){
    	colWidth = w;
    }
      
    public void paint(Graphics g) {
        int width = getWidth();
        int height = getHeight();
        g.setColor(Color.white);
        g.fillRect(0, 0, width, height);
        g.setColor(Color.gray);
        g.drawLine(0, getHeight()-1, width, getHeight()-1);
        int x = 0;
        int thresholdPos = (int)(height - ((height)*threshold));
        
        for (Iterator it=entryList.iterator(); it.hasNext(); ){ 
        	ChartEntry entry = (ChartEntry)it.next();
        	AttributeValue value =(AttributeValue)entry.map.get(attributeName);
        	int h = 0;
        	int hpos = height;
        	if (value != null){ 
        		h = (int)(value.weight()*height);
        		hpos = height - h;
        	}
        	g.setColor(color);
        	int wthresh = Math.max(hpos, thresholdPos);        	
        	if (wthresh < height){
        		g.setColor(Color.darkGray);
        		g.fillRect(x, wthresh, colWidth, h);
        	}
        	if (wthresh > hpos){ 
        		g.setColor(color);
        		g.fillRect(x, hpos, colWidth, wthresh - hpos);
        	}
          
        	g.setColor(Color.WHITE);
        	g.fillRect(x, 0, colWidth, (int)((1-value.weight())*height));
                	
        	g.setFont(new Font("Arial", Font.PLAIN, 11));
        	
        	try{
	          	if(entry.getShowFlag()){
	          		g.setColor(Color.darkGray);
	          		g.drawString(value.stringValue(), x+2, wthresh-5);
	          	}
        	}catch(java.lang.NullPointerException ne){}
          
	
        	g.setColor(Color.lightGray);
        	x += colWidth;
        	g.drawLine(x-1, 0, x-1, height - 1);    
          
        	if (x > width){ 
        		break;
        	}
        }
        
        g.setColor(Color.lightGray);
        g.drawLine(0, height-1, width-1, height-1);
        if (thresholdPos > 0){
        	g.setColor(Color.darkGray);
        	g.drawLine(0, thresholdPos, width-1, thresholdPos);
        }
    }
 
    private class NullSelectionModel implements SingleSelectionModel {
        public void addChangeListener(ChangeListener listener) {
        }
        
        public void clearSelection() {
        }
        
        public int getSelectedIndex(){ 
        	return -1;
        }
        
        public boolean isSelected(){
        	return false;
        }
        
        public void removeChangeListener(ChangeListener listener) {
        }
        
        public void setSelectedIndex(int index) {
        }
    }    
    
    private class ResizeHandler extends ComponentAdapter {
        public void componentResized(ComponentEvent e)
        { getHeight();
        }
    }

    private class MouseHandler extends MouseInputAdapter {
        private final int OFF = 0;
        private final int NEARBY = 1;
        private final int MOVING = 2;
        private int dragMode = OFF;
        private int dragY;//-
        private int thresholdPos = 0;
        
        Cursor movingCursor = new Cursor(Cursor.MOVE_CURSOR);
        Cursor defaultCursor = null;
        boolean nearThreshold = false;
        
        public void mouseDragged(MouseEvent e) {
            int newY = e.getY() + getLocation().y; 
            int delY = newY - dragY;
            dragY = newY;
            int height = getHeight();
            
            if (dragMode == MOVING && height > 1){ 
            	thresholdPos += delY;
            	if (thresholdPos < 0){ 
            		thresholdPos = 0;
            	}
            	else if (thresholdPos > height-1){ 
            		thresholdPos = height-1;
            	}
            	threshold = (height - thresholdPos)/(double)(height-1);//-
            	boolean redisplay = false;
            	for (Iterator it=entryList.iterator(); it.hasNext(); ){ 
            		ChartEntry entry = (ChartEntry)it.next();
            		AttributeValue value = (AttributeValue)entry.map.get(attributeName);
            		double h = (value != null ? value.weight() : 0);
            		if (threshold >= h && !entry.isMasked() && threshold > 0.02){//added to fix bug0 
            			if (entry.addMaskingAttribute(AttributeCell.this)){ 
            				redisplay = true;
            			}
            		}
            		else if (threshold < h && entry.isMasked() || threshold < 0.02){//added to fix bug0 
            			if (entry.removeMaskingAttribute(AttributeCell.this)){ 
            				redisplay = true;
            			}
            		}
            	}
            	if (redisplay){ 
            		chart.updateDisplay();
            	}
            	repaint();
            }
        }
        
        public void mouseMoved(MouseEvent e) {
        	int y = e.getY();
            if (defaultCursor == null){ 
            	defaultCursor = getCursor();
            }
            thresholdPos = (int)(getHeight() - (threshold*(getHeight()- 1)));//-
            if (y >= thresholdPos - 2&& y <= thresholdPos){ 
            	if (dragMode == OFF){ 
            		dragMode = NEARBY;
            		setCursor(movingCursor);
            	}
            }
            else{ 
            	if (dragMode == NEARBY){ 
            		dragMode = OFF;
            		setCursor(defaultCursor);
            	}
            }
        }
        
        public void mouseExited(MouseEvent e) {
            if (dragMode == NEARBY){ 
            	dragMode = OFF;
            	setCursor(defaultCursor);
            }
        }
        
        public void mousePressed(MouseEvent e){ 
        	if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) != 0){ 
        		if (dragMode == NEARBY){ 
        			dragMode = MOVING;
        		}
        		dragY = e.getY() + getLocation().y;
        	}
        	else if ((e.getModifiers() & MouseEvent.BUTTON2_MASK) != 0){ 
        		int index = e.getX()/colWidth;
        		if (index >= 0 && index < entryList.size()){ 
        			ChartEntry entry = (ChartEntry)entryList.get(index);
        			AttributeValue value = (AttributeValue)entry.map.get(attributeName);
        			String msg = "undefined";
        			if (value != null){ 
        				msg = value.stringValue();
        				if (units != null){ 
        					msg += " " + units;
        				}
        			}
        			JPopupMenu popup = getEntryPopup();
        			entryPopupMenuItem.setText(msg);
        			Point loc = AttributeCell.this.getLocationOnScreen();
        			popup.setLocation(loc.x+(index+1)*colWidth-5,loc.y+e.getY());
        			popup.setVisible(true);
        		}
        	}          
        }
        
        public void mouseReleased(MouseEvent e){ 
        	if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) != 0){ 
        		if (dragMode == MOVING){ 
        			dragMode = OFF;
        			setCursor(defaultCursor);
        		}
        	}
        	else if ((e.getModifiers() & MouseEvent.BUTTON2_MASK) != 0){ 
        		JPopupMenu popup = getEntryPopup();
        		if (popup.isVisible()){ 
        			popup.setVisible(false);
        		}
        	}
        }
    }
}
