import java.awt.*;

import javax.swing.*;
import java.util.*;

//This class is the overall ranking panel. The most important function in this
//class is the paint function. All the rankings are done through paint.
//If you are rotating the interface 90 degrees, you need to change the paint function
//of x to y.

//See below comments

public class DisplayPanel extends JComponent{
	private static final long serialVersionUID = 1L;
	boolean score = false;
	boolean ruler = false;
	private TablePane rootPane;
	private int colWidth = ValueChart.DEFAULT_COL_WIDTH;
	private Vector entryList;

	public DisplayPanel(int col){
		colWidth = col;
	}
	
	public int getColWidth(){ 
		return colWidth;
	}

/*	public void setColWidth (int w){ 
		colWidth = w;
		}
*/
	public void setRootPane (TablePane pane){
		rootPane = pane;
	}

	public void setEntries (Vector list){
		entryList = list;
		Dimension dim = getPreferredSize();
		
		dim.width = colWidth*entryList.size();
		setPreferredSize(dim);	
		setMaximumSize(new Dimension(colWidth*entryList.size()+colWidth*2, 10000));	
	}

	public Dimension getPreferredSize(){
		return new Dimension (colWidth*entryList.size()+colWidth*2, getHeight());
	}

	public void setScore(boolean s){
		score = s;
	}
	
	public void setRuler(boolean r){
		ruler = r;
	}
	
	public void paint(Graphics g){
		int numEntries = entryList.size();	   
		int totalWidth = getWidth();
		int totalHeight = getHeight();
		g.setColor (Color.white);
		int align_right = 0;//totalWidth - numEntries*colWidth;//
		g.fillRect (align_right + colWidth, 0, numEntries*colWidth, totalHeight);//
		if (rootPane == null){ 
			return; 
	    }

		Vector cellList = new Vector(16);
		rootPane.getAttributeCells (cellList);
		double[] weights;	//array of entry weights (values)
		double[] accumulatedRatios = new double[numEntries]; 
		int[] ypos = new int[numEntries];	//position of x, starts all at 0
		//for each primobj
		for (Iterator it=cellList.iterator(); it.hasNext(); ){
			//for each objective, get the (array)domain value of each alternative
			AttributeCell cell = (AttributeCell)it.next();			
			weights = cell.getWeights();
			//absolute weight
			double or = cell.getOverallRatio();
			int h, x = align_right + colWidth;
			//for each entry, paint the said attribute's value
			for (int i=0; i<numEntries; i++){ 				
				accumulatedRatios[i] += or*weights[i];				
				h = (int)Math.round(accumulatedRatios[i]*totalHeight)-ypos[i];				
				ChartEntry entry = (ChartEntry)entryList.get(i);
				if (entry.isMasked()){ 
					g.setColor (Color.lightGray);
				}
				else{ 
					g.setColor (cell.getColor());	  
				}
				
				g.fillRect (x, getHeight() - h - ypos[i], colWidth, h);
                g.setColor (Color.lightGray);
                g.drawLine (x, getHeight() - h - ypos[i], x+colWidth-1, getHeight() - h - ypos[i]);
                x += colWidth;
                ypos[i] += h;
                if (x > totalWidth){ 
                	break;
                }               
			}
		}
		int y = colWidth + (colWidth/4);
		
		//scores
		if(score)
			for (int j=0; j<numEntries; j++){
				g.setColor(Color.darkGray);
				g.drawString(String.valueOf((Math.round(accumulatedRatios[j] * 100))), y, totalHeight - ypos[j] -5);
				y += colWidth;
			}  
		
		g.setColor (Color.lightGray);
		y = align_right + colWidth;	//temp, was only colWidth
		for (int i=1; i<numEntries + 1; i++){ 
		   	g.drawLine (y, 0, y, totalHeight-1);
		    y += colWidth;
		}
		
		//ruler
		if (ruler){
			g.setColor (Color.gray);
			g.drawRect(1, 0, colWidth - 2, totalHeight-1);
			
			for (int i=1; i<10; i++){
				g.drawLine(colWidth - 5, i*totalHeight/10, colWidth - 2, i*totalHeight/10);
				g.setFont(new Font("Arial", Font.PLAIN, 8));	
				g.setColor (Color.darkGray);
				g.drawString(String.valueOf((10-i) * 10), colWidth - 12, i*totalHeight/10 + 3);				
			}
		}
		boolean gridline = false;
		if (gridline){
			g.setColor (Color.lightGray);
			for (int i=1; i<10; i++){
				g.drawLine(i*totalWidth/10, colWidth - 2, i*totalWidth/10, totalHeight);
			}
		}
	 }
}
